# {FileStr} is library that augments String objects with File, Dir,
# and FileUtils capabilities (class methods). The String (self) is
# passed as the first argument to original class methods.


require          'fileutils'


# {FileStr} class related module which maps missing methods to
# {FileStr} defined methods.
module FileStrModule

    # Call FileStr supported methods (if available).
    def method_missing( m, *args, &block )

        klass = FileStr.support[ m ]

        if klass
            # Generate the actual class method name.
            cm = m.to_s[2..-1].to_sym
            klass.send( cm, self, *args, &block )
        else
            raise NoMethodError
        end
    end


    # ------------------------------------------------------------
    # File class support:

    # Chmod with mode_int.
    #
    # @param mode_int [Integer] New file mode.
    def s_chmod( mode_int )
        File.chmod( mode_int, self )
    end

    # Chown with owner_int, group_int.
    #
    # @param owner_int [Integer] New file owner.
    # @param group_int [Integer] New file group.
    def s_chown( owner_int, group_int )
        File.chown( owner_int, group_int, self )
    end

    # Fnmatch to pattern with flags.
    #
    # @param pattern [String]   Glob patttern for matching.
    # @param flags   [Constant] Flags for matching.
    def s_fnmatch( pattern, *flags )
        File.fnmatch( pattern, self, *flags )
    end


    # ------------------------------------------------------------
    # Dir class support:

    # Glob with self as base string.
    #
    # @param pattern [String] Suffix of glob pattern (concat to self).
    def s_glob( pattern = "", *args, &block )
        Dir.glob( self + pattern, *args, &block )
    end

end


# Enable use of File, Dir, and FileUtils class methods straight from
# {FileStr}/String objects.
class FileStr < String

    include FileStrModule


    # Extend a class (String) with {FileStr} features.
    #
    # @param klass [Constant] Class to extend.
    def FileStr.extend( klass = String )
        klass.send( :include, FileStrModule )
    end

    # Return hash describing {FileStr} provided File/Dir support.
    def FileStr.support
        @@fileStrSupport
    end

    # Supported methods from File, Dir, and FileUtils.
    @@fileStrSupport = {

        # File Methods from IO.
        :f_binread          => File,
        :f_binwrite         => File,
        :f_copy_stream      => File,
        # :for_fd => File, # NA
        :f_foreach          => File,
        # :new => File,    # NA
        # :open => File,   # NA
        # :pipe => File,   # NA
        # :popen => File,  # NA
        :f_read             => File,
        :f_readlines        => File,
        # :select => File, # NA
        :f_sysopen          => File,
        # :try_convert => File, # NA
        :f_write            => File,

        # File Methods.
        :f_absolute_path    => File,
        :f_atime            => File,
        :f_basename         => File,
        :f_birthtime        => File,
        :f_blockdev?        => File,
        :f_chardev?         => File,
        # :chmod => File, # s-version
        # :chown => File, # s-version
        :f_ctime            => File,
        :f_delete           => File,
        :f_directory?       => File,
        :f_dirname          => File,
        :f_executable?      => File,
        :f_executable_real? => File,
        :f_exist?           => File,
        :f_exists?          => File,
        :f_expand_path      => File,
        :f_extname          => File,
        :f_filename         => File,
        :f_file?            => File,
        # :fnmatch => File,  # s-version
        # :fnmatch? => File, # NA
        :f_ftype            => File,
        :f_grpowned?        => File,
        :f_identical?       => File,
        :f_join             => File,
        # :lchmod => File,
        # :lchown => File,
        :f_link             => File,
        :f_lstat            => File,
        # :mkfifo => File, # Unknown
        :f_mtime            => File,
        :f_new              => File,
        :f_open             => File,
        :f_owned?           => File,
        :f_path             => File,
        :f_pipe?            => File,
        :f_readable?        => File,
        :f_readable_real?   => File,
        :f_readlink         => File,
        :f_realdirpath      => File,
        :f_realpath         => File,
        :f_rename           => File,
        :f_setgid?          => File,
        :f_setuid?          => File,
        :f_size             => File,
        :f_size?            => File,
        :f_socket?          => File,
        :f_split            => File,
        :f_stat             => File,
        :f_sticky?          => File,
        :f_symlink          => File,
        :f_symlink?         => File,
        :f_truncate         => File,
        # :umask => File, # NA
        :f_unlink           => File,
        # :utime => File, # NA
        :f_world_readable?  => File,
        :f_world_writable?  => File,
        :f_writable?        => File,
        :f_writable_real?   => File,
        :f_zero?            => File,

        # Dir Methods.
        :d_chdir            => Dir,
        :d_chroot           => Dir,
        :d_delete           => Dir,
        :d_entries          => Dir,
        :d_exist?           => Dir,
        :d_exists?          => Dir,
        :d_foreach          => Dir,
        # :getwd => Dir, # NA
        # :glob => Dir,  # s-version
        # :home => Dir,  # NA
        :d_mkdir            => Dir,
        :d_new              => Dir,
        :d_open             => Dir,
        # :pwd => Dir,   # NA
        :d_rmdir            => Dir,
        :d_unlink           => Dir,

        # FileUtils Methods (selection).
        :u_cd               => FileUtils,
        :u_cmp              => FileUtils,
        :u_mkdir_p          => FileUtils,
        :u_ln_s             => FileUtils,
        :u_cp               => FileUtils,
        :u_cp_r             => FileUtils,
        :u_mv               => FileUtils,
        :u_rm               => FileUtils,
        :u_rm_f             => FileUtils,
        :u_rm_r             => FileUtils,
        :u_rm_rf            => FileUtils,
        :u_touch            => FileUtils,
        :u_uptodate?        => FileUtils,
    }

end



# Extend String with capability to convert itself to {FileStr}.
class String

    # Use FileStr method directly in String.
    def String.fileStr
        FileStr.extend( String )
    end

    # Convert String to FileStr object.
    def toFileStr
        FileStr.new( self )
    end

    alias to_fs toFileStr

    alias fs toFileStr

end

require_relative 'version'
